SystemOrganization addCategory: #NBSQLite3!
SystemOrganization addCategory: 'NBSQLite3-Core'!
SystemOrganization addCategory: 'NBSQLite3-Tests'!

TestCase subclass: #NBSQLite3BaseTest
	instanceVariableNames: 'db db2 res res2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Tests'!

!NBSQLite3BaseTest commentStamp: 'PierceNg 3/29/2014 20:18' prior: 0!
I provide the test cases for NBSQLite3BaseConnection.!

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:21'!
setUp 

	super setUp.
	db := NBSQLite3BaseConnection on: ':memory:'.
	db open.
! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 4/5/2014 13:26'!
tearDown

	super tearDown.
	[ db close ] on: NBSQLite3Misuse do: [ ]
! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/29/2014 19:59'!
testColumnNames
	| columns |
	
	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	columns := db columnNamesFor: 'x'.
	self assert: (columns size = 2).
	self assert: (columns first = 'xk').
	self assert: (columns second = 'xv').
	
! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/29/2014 20:11'!
testColumnNamesBobbyGo
	"In this test, Bobby Tables strikes naive SQL string construction."
	| tables bobby |
	
	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db execute: 'create table students (sk integer, sname varchar);'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	

	bobby := 'x); drop table students; --'.	
	res := db execute: 'pragma table_info(', bobby, ')'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 1).
	self assert: (tables first = 'x').	

! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:29'!
testColumnNamesBobbyStop
	"In this test, Bobby Tables strikes again, but fails, because #columnNamesFor: uses parameter binding."
	| columns tables |
	
	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db execute: 'create table students (sk integer, sname varchar);'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	
	
	columns := db columnNamesFor: 'x; drop table students; --'.
	self assert: (columns isNil).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	

! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/29/2014 20:23'!
testColumnNamesNoSuchTable
	| columns |
	
	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	columns := db columnNamesFor: 'y'.
	self assert: (columns isNil).! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:23'!
testDataValuesAvailable
	| s | 
	
	res := db execute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db execute: 'insert into x values (NULL, 1, "duh");'.	
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s execute: [  :row |
		self assert: (row dataValuesAvailable = 3) ].
	s finalize.
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 4/5/2014 13:26'!
testDoubleClose
	
	db close.
	"#teardown will close the db again."! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 2/24/2014 21:28'!
testExecuteOk

	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db execute: 'drop table x;'.
	self assert: (res = 0).
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 2/24/2014 20:47'!
testExecuteSyntaxError

	self should: [ db execute: 'create table,;' ]
		raise: NBSQLite3Error
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 17:07'!
testFloat
	| s |
	
	res := db execute: 'create table x (xk integer primary key, xv real);'.
	self assert: (res = 0).
	res := db execute: 'insert into x values (NULL, 1.141);'.
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s execute: [ :row |
		self assert: ((row nameOfColumn: 1) = 'xv').
		self assert: ((row floatAt: 1) = 1.141) ].
	s finalize.
! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 17:57'!
testObjectBlob
	| data s | 
	
	data := OrderedCollection with: 1 with: 'two' with: 3.3.
	
	res := db execute: 'create table x (xk integer primary key, xv blob);'.
	self assert: (res = 0).

	res := db execute: 'begin'.
	self assert: (res = 0).
	s := db prepare: 'insert into x values (NULL, ?)'.
	s at: 1 putObject: data.
	s step. s finalize.
	res := db execute: 'commit'.
	self assert: (res = 0).
	
	s := db prepare: 'select xv from x'.
	s execute: [  :row |
		| v |
		v := row objectAt: 0.
		self assert: ((v size) = 3).
		self assert: ((v first) = 1).
		self assert: ((v second) = 'two').
		self assert: ((v third) = 3.3) ].
	s finalize.
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:23'!
testPrepareStep
	| s i | 
	
	res := db execute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db execute: 'begin'.
	self assert: (res = 0).
	s := db prepare: 'insert into x values (NULL, ?, ?)'.
	1 to: 10 do: [ :x |
		s at: 1 putInteger: x * x.
		s at: 2 putString: x asString, ' * ', x asString.
		s step. s clearBindings. s reset. ].		
	s finalize.
	res := db execute: 'commit'.
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s execute: [  :row |
		i := row integerAt: 0.
		self assert: ((row integerAt: 1) = (i * i)).
		self assert: ((row stringAt: 2) = (i asString, ' * ', i asString)) ].
	s finalize.
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 17:19'!
testResultSetColumnIntrospection
	| s | 

	res := db execute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db execute: 'insert into x values (NULL, 1, "duh");'.	
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s execute: [  :row |
		self assert: ((row nameOfColumn: 0) = 'xk').
		self assert: ((row valueOfColumn: 0) = 1).
		self assert: ((row nameOfColumn: 1) = 'iv').
		self assert: ((row valueOfColumn: 1) = 1).
		self assert: ((row nameOfColumn: 2) = 'tv').
		self assert: ((row valueOfColumn: 2) = 'duh') ].
	s finalize.
	! !

!NBSQLite3BaseTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:23'!
testTableNames
	| tables |
	
	res := db execute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	tables := db tableNames.
	self assert: (tables size = 1).
	self assert: (tables first = 'x')
	
! !

Error subclass: #NBSQLite3Error
	instanceVariableNames: 'resultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Error commentStamp: '<historical>' prior: 0!
I represent an SQLite non-ok condition.!

NBSQLite3Error subclass: #NBSQLite3Busy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Busy commentStamp: '<historical>' prior: 0!
I represent the SQLITE_BUSY condition.!

!NBSQLite3Error methodsFor: 'accessing' stamp: 'PierceNg 2/27/2014 22:11'!
resultCode
	^ resultCode! !

!NBSQLite3Error methodsFor: 'accessing' stamp: 'PierceNg 2/27/2014 22:11'!
resultCode: anObject
	resultCode := anObject! !

NBSQLite3Error subclass: #NBSQLite3Locked
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Locked commentStamp: '<historical>' prior: 0!
I represent the SQLITE_LOCKED condition.!

NBSQLite3Error subclass: #NBSQLite3Misuse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

NBSQLite3Error subclass: #NBSQLite3NotOpened
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3NotOpened commentStamp: 'PierceNg 3/29/2014 20:17' prior: 0!
I am signalled when operations are invoked on a database that is not open.
!

Object subclass: #NBSQLite3BaseConnection
	instanceVariableNames: 'dbFilename dbHandle isOpen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3BaseConnection commentStamp: '<historical>' prior: 0!
I represent a connection to an SQLite database.!

!NBSQLite3BaseConnection class methodsFor: 'instance finalization' stamp: 'PierceNg 4/5/2014 13:05'!
finalizeResourceData: dbHandle
	NBSQLite3FFI current apiClose: dbHandle.
	! !

!NBSQLite3BaseConnection class methodsFor: 'instance creation' stamp: 'PierceNg 7/7/2013 20:34'!
on: aFilename
	^ self new on: aFilename
! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:48'!
clearBindings: aStatement
	NBSQLite3FFI current clearBindings: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 3/29/2014 19:39'!
close

	NBSQLite3FFI current close: dbHandle.
	isOpen := false! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 3/29/2014 19:53'!
columnNamesFor: aTableName
	
	self isOpen
		ifFalse: [ NBSQLite3NotOpened signal ]
		ifTrue: [ 
			| s tableExists |
			s := self prepare: 'select count(*) from sqlite_master where type = "table" and tbl_name = ?'.
			s at: 1 putString: aTableName.
	  		s execute: [ :row | tableExists := ((row integerAt: 0) = 1) ].
	  		s finalize.
			tableExists ifTrue: [ 
				| columns |
				columns := OrderedCollection new.
				s := self prepare: 'pragma table_info(', aTableName, ')'.
				s execute: [ :row | columns add: (row stringAt: 1) ].
				s finalize.
				^ columns ].
			^ nil ]
		! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 2/15/2014 20:41'!
execute: anSQLText 
	^ NBSQLite3FFI current execute: anSQLText on: dbHandle.
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/3/2014 15:54'!
finalize: aStatement
	NBSQLite3FFI current finalize: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'initializing' stamp: 'PierceNg 1/3/2014 14:49'!
initialize 

	dbHandle := NBExternalObject new.
	isOpen := false.! !

!NBSQLite3BaseConnection methodsFor: 'testing' stamp: 'PierceNg 3/29/2014 19:40'!
isOpen
	^ isOpen! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 3/29/2014 20:19'!
on: aFilename

	dbFilename := aFilename.
	^ self! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 09:20'!
open

	NBSQLite3FFI current open: dbFilename via: dbHandle.
	isOpen := true.
	^ self! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/3/2014 15:55'!
prepare: anSQLText 
	| s |
	
	s := NBSQLite3Statement new.
	s connection: self.
	NBSQLite3FFI current prepare: s handle on: dbHandle with: anSQLText.
	^ s
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:29'!
reset: aStatement
	NBSQLite3FFI current reset: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:11'!
step: aStatement
	NBSQLite3FFI current step: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 3/29/2014 19:42'!
tableNames
	| tables s |
	
	self isOpen
		ifFalse: [ NBSQLite3NotOpened signal ]
		ifTrue: [ 
			tables := OrderedCollection new.
			s := self prepare: 'select tbl_name from sqlite_master where type = "table"'.
	  		s execute: [ :row | tables add: (row stringAt: 0) ].
	  		s finalize.
			^ tables ]! !

Object subclass: #NBSQLite3Constants
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Constants commentStamp: '<historical>' prior: 0!
I provide SQLIte's constants.!

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
BLOB
	^ 4! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 2/27/2014 22:17'!
BUSY
	^ 5! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
FLOAT
	^ 2! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
INTEGER
	^ 1! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 2/27/2014 22:17'!
LOCKED
	^ 6! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 4/5/2014 13:21'!
MISUSE
	^ 21! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
NULL
	^ 5! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 4/5/2014 12:40'!
OK
	^ 0! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
TEXT
	^ 3! !

Object subclass: #NBSQLite3Example
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Example commentStamp: '<historical>' prior: 0!
I implement some examples of using this package.!

!NBSQLite3Example class methodsFor: 'examples' stamp: 'PierceNg 2/27/2014 23:07'!
multithreading
	self new multithreading! !

!NBSQLite3Example methodsFor: 'examples' stamp: 'PierceNg 2/28/2014 23:37'!
multithreading
	| db db2 sem blocking blocked |
	
	db := NBSQLite3BaseConnection on: 'file::memory:?cache=shared'.
	db2 := NBSQLite3BaseConnection on: 'file::memory:?cache=shared'.
	sem := Semaphore new.	

	Transcript open; clear.	
	
	blocking :=	[ :x :sema |
		Transcript show: 'Thread-1: begin exclusive...'; cr; flush.
		[ x open.
			x execute: 'begin exclusive'.
			sema wait.
			Transcript show: 'Thread-1: releasing exclusive lock.'; cr; flush.
			x execute: 'rollback' 
		] ensure: [ x close ] 
	] newProcessWith: (Array with: db with: sem).

	blocking resume.
	Processor yield.
	
	blocked := [ :x |
		Transcript show: 'Thread-2: create table ...'; cr; flush.
		[[ x open. x execute: 'create table if not exists x (xk integer)' ]
		on: NBSQLite3Locked do: [ :ex | Transcript show: ex messageText; cr; flush ]
		] ensure: [ x close ]
	] newProcessWith: (Array with: db2).
	
	blocked resume.
	Processor yield.

	sem signal.
! !

Object subclass: #NBSQLite3FFI
	instanceVariableNames: ''
	classVariableNames: 'DataTypeMap Singleton TypeMap'
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3FFI commentStamp: '<historical>' prior: 0!
I am the FFI interface to SQLite.!

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 1/3/2014 14:46'!
current
	^ Singleton ifNil: [ Singleton := self basicNew ]! !

!NBSQLite3FFI class methodsFor: 'class initialization' stamp: 'PierceNg 3/30/2014 17:09'!
initialize

	self initializeTypeMap.
	self initializeDataTypeMap.
! !

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 3/30/2014 17:09'!
initializeDataTypeMap

	DataTypeMap := Dictionary newFromPairs: #(
		1	#integerAt: 
		2	#floatAt:
		3	#stringAt:
		4	#byteArrayAt:
		5	#nilAt:
	)! !

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 2/16/2014 00:31'!
initializeTypeMap

	TypeMap := Dictionary newFromPairs: #(
		sqlite3		      NBExternalObject 
		sqlite3_stmt	NBExternalObject
		sqlite_int64	NBInt64
	)! !

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 1/3/2014 12:35'!
nbBindingOf: aTypeName
	^ TypeMap at: aTypeName ifAbsent: [ super nbBindingOf: aTypeName ]! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/23/2014 20:10'!
apiBindBlob: aStatement atColumn: aColumn with: aByteArray with: anInteger with: aPointer
	"int sqlite3_bind_blob(sqlite3_stmt*, int, const void *, int, void(*)(void*))"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_blob (sqlite3_stmt aStatement, int aColumn, ByteArray aByteArray, int anInteger, int aPointer)) "SQLITE_TRANSIENT"
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 17:06'!
apiBindDouble: aStatement atColumn: aColumn with: aFloat
	"int sqlite3_bind_double(sqlite3_stmt*, int, double)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_double (sqlite3_stmt aStatement, int aColumn, double aFloat))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/16/2014 00:31'!
apiBindInteger: aStatement atColumn: aColumn with: anInteger
	"int sqlite3_bind_int64(sqlite3_stmt*, int, sqlite_int64)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_int64 (sqlite3_stmt aStatement, int aColumn, sqlite_int64 anInteger))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:42'!
apiBindString: aStatement atColumn: aColumn with: aString with: anInteger with: anotherInteger
	"int sqlite3_bind_text(sqlite3_stmt*, int, const char*, int, void(*)(void*))"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_text (sqlite3_stmt aStatement, int aColumn, String aString, int anInteger, int anotherInteger))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:35'!
apiClearBindings: aStatement
	"int sqlite3_clear_bindings(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_clear_bindings (sqlite3_stmt aStatement))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:07'!
apiClose: handle
	"int sqlite3_close_v2(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_close_v2 (sqlite3 handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/23/2014 20:12'!
apiColumnBlob: aStatement atColumn: aColumn
	"const void *sqlite3_column_blob(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(const void *sqlite3_column_blob (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 22:42'!
apiColumnBytes: aStatement atColumn: aColumn
	"int sqlite3_column_bytes(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_bytes (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 17:07'!
apiColumnDouble: aStatement atColumn: aColumn
	"double sqlite3_column_double(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(double sqlite3_column_double (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:31'!
apiColumnInt: aStatement atColumn: aColumn
	"int sqlite3_column_int(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_int (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 16:08'!
apiColumnName: aStatement atColumn: aColumn
	"const char *sqlite3_column_name(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_column_name (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 22:41'!
apiColumnText: aStatement atColumn: aColumn
	"const unsigned char *sqlite3_column_text(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_column_text (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 16:07'!
apiColumnType: aStatement atColumn: aColumn
	"int sqlite3_column_type(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_type (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 15:47'!
apiDataCount: handle
	"int sqlite3_data_count(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_data_count (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:15'!
apiErrorMessage: handle 
	"const char *sqlite3_errmsg(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_errmsg (sqlite3 handle))
		module: self library	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/15/2014 20:45'!
apiExec: dbHandle withSQLText: anSQLText with: callback with: callbackData with: errorMessageBuffer
	"int sqlite3_exec(sqlite3*, const char*, int (*callback), void*, char **)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_exec (sqlite3 dbHandle, String anSQLText, void *callback, void *callbackData, char **errorMessageBuffer))
		module: self library
	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:06'!
apiFinalize: handle
	"int sqlite3_finalize(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_finalize (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:14'!
apiOpen: filename via: handle
	"int sqlite3_open(const char*, sqlite3**)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_open (String filename, sqlite3* handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:25'!
apiPrepare: dbHandle withSQLText: anSQLText with: textSize with: sHandle with: tail
	"int sqlite3_prepare_v2(sqlite3*, const char*, int, sqlite3_stmt**, const char **)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_prepare_v2 (sqlite3 dbHandle, String anSQLText, int textSize, sqlite3_stmt* sHandle, const char** tail))
		module: self library
	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:13'!
apiReset: handle
	"int sqlite3_reset(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_reset (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:12'!
apiStep: handle
	"int sqlite3_step(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_step (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/26/2014 21:14'!
apiThreadsafe
	"int sqlite3_threadsafe(void)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_threadsafe ())
		module: self library
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/23/2014 20:14'!
blobFrom: aStatement at: aColumn
	| address size |
	
	address := self apiColumnBlob: aStatement atColumn: aColumn.
	size := self apiColumnBytes: aStatement atColumn: aColumn.
	^ (((NBExternalArray ofType: NBCharacterType) onAddress: address size: size) postCopy; yourself) address! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 4/5/2014 13:22'!
checkForOk: aValue on: aHandle

	(aValue = 0)
		ifFalse: [ 
			(aValue = NBSQLite3Constants BUSY)
				ifTrue: [ self signal: NBSQLite3Busy with: aValue on: aHandle ].
			(aValue = NBSQLite3Constants LOCKED)
				ifTrue: [ self signal: NBSQLite3Locked with: aValue on: aHandle ].
			(aValue = NBSQLite3Constants MISUSE)
				ifTrue: [ self signal: NBSQLite3Misuse with: aValue on: aHandle ].				
			self signal: NBSQLite3Error with: aValue on: aHandle ].	
	^ 0! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
clearBindings: sHandle on: dbHandle	
	self checkForOk: (self apiClearBindings: sHandle) on: dbHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
close: aHandle	
	self checkForOk: (self apiClose: aHandle) on: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:10'!
columnNameFrom: aStatement at: aColumn
	^ self apiColumnType: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:10'!
columnTypeFrom: aStatement at: aColumn
	^ self apiColumnType: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 15:55'!
dataValuesAvailable: aStatementHandle
	^ self apiDataCount: aStatementHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/24/2014 20:44'!
execute: anSQLText on: aDBHandle
	| null |
	
	null := NBExternalAddress null.
	^ self checkForOk: (self apiExec: aDBHandle 
							withSQLText: anSQLText
							with: null
							with: null
							with: null)
		 on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
finalize: aStatementHandle on: aDBHandle	
	self checkForOk: (self apiFinalize: aStatementHandle) on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:54'!
floatFrom: aStatement at: aColumn
	^ self apiColumnDouble: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:53'!
integerFrom: aStatement at: aColumn
	^ self apiColumnInt: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'accessing' stamp: 'PierceNg 1/3/2014 14:01'!
library
	| platform |
	
	platform := NativeBoost forCurrentPlatform class.
	platform = NativeBoostMac32 ifTrue: [ ^ #sqlite3 ].
	^ 'sqlite3'! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:12'!
nameFor: aStatement at: aColumn
	^ self apiColumnName: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
open: aFilename via: aHandle	
	self checkForOk: (self apiOpen: aFilename via: aHandle) on: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
prepare: aStatementHandle on: aDBHandle with: anSQLText

	self checkForOk: (self apiPrepare: aDBHandle 
							withSQLText: anSQLText
							with: (anSQLText size)
							with: aStatementHandle
							with: (NBExternalAddress null))
		 on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:23'!
reset: aStatementHandle on: aDBHandle	
	self checkForOk: (self apiReset: aStatementHandle) on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/27/2014 22:21'!
signal: exceptionClass with: value on: handle
	exceptionClass new resultCode: value; signal: (self apiErrorMessage: handle)! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:22'!
step: aStatementHandle	
	^ self apiStep: aStatementHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:52'!
stringFrom: aStatement at: aColumn
	^ self apiColumnText: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'accessing' stamp: 'PierceNg 2/26/2014 21:15'!
threadsafe
	^ self apiThreadsafe! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:39'!
typeFor: aStatement at: aColumn
	^ DataTypeMap at: (self apiColumnType: aStatement atColumn: aColumn)! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/23/2014 16:15'!
with: aStatement at: aColumn putBlob: aByteArray
	^ self apiBindBlob: aStatement atColumn: aColumn with: aByteArray with: aByteArray size with: -1 ! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:57'!
with: aStatement at: aColumn putFloat: aFloat
	^ self apiBindDouble: aStatement atColumn: aColumn with: aFloat! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:39'!
with: aStatement at: aColumn putInteger: anInteger
	^ self apiBindInteger: aStatement atColumn: aColumn with: anInteger   ! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:41'!
with: aStatement at: aColumn putString: aString
	^ self apiBindString: aStatement atColumn: aColumn with: aString with: -1 with: -1 ! !

Object subclass: #NBSQLite3Statement
	instanceVariableNames: 'connection handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Statement commentStamp: '<historical>' prior: 0!
I represent an SQLite prepared statement.!

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:49'!
at: aColumn putByteArray: anObject
	^ NBSQLite3FFI current with: handle at: aColumn putBlob: anObject! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/16/2014 00:25'!
at: aColumn putInteger: anInteger
	^ NBSQLite3FFI current with: handle at: aColumn putInteger: anInteger! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/23/2014 16:21'!
at: aColumn putObject: anObject
	^ NBSQLite3FFI current with: handle at: aColumn putBlob: (FLSerializer serializeToByteArray: anObject)! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/16/2014 00:26'!
at: aColumn putString: aString
	^ NBSQLite3FFI current with: handle at: aColumn putString: aString! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:49'!
byteArrayAt: aColumn
	^ NBSQLite3FFI current blobFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 21:47'!
clearBindings
	connection clearBindings: self! !

!NBSQLite3Statement methodsFor: 'private' stamp: 'PierceNg 1/3/2014 15:50'!
connection: anObject
	connection := anObject! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 15:55'!
dataValuesAvailable
	^ NBSQLite3FFI current dataValuesAvailable: handle! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 21:11'!
execute: aBlock
	| ret |
	
	ret := self step. 
	(ret = 101) " == SQLITE_DONE, meaning step should not be sent again."
		ifTrue: [ aBlock value: self ]
		ifFalse: [
			[ ret = 100 ] whileTrue: [ " == SQLITE_ROW, meaning another row is available."
				aBlock value: self.
				ret := self step ]]! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/3/2014 16:49'!
finalize 
	connection finalize: self! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 17:05'!
floatAt: aColumn
	^ NBSQLite3FFI current floatFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'private' stamp: 'PierceNg 1/3/2014 16:31'!
handle
	^ handle! !

!NBSQLite3Statement methodsFor: 'initializing' stamp: 'PierceNg 1/3/2014 16:30'!
initialize

	handle := NBExternalObject new.! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:00'!
integerAt: aColumn
	^ NBSQLite3FFI current integerFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:16'!
nameOfColumn: aColumn
	^ NBSQLite3FFI current nameFor: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:50'!
nilAt: aColumn
	^ nil! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/23/2014 16:20'!
objectAt: aColumn
	^ FLMaterializer materializeFromByteArray: (NBSQLite3FFI current blobFrom: handle at: aColumn)! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 22:30'!
reset
	connection reset: self! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:28'!
step
	^ NBSQLite3FFI current step: handle! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:00'!
stringAt: aColumn
	^ NBSQLite3FFI current stringFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:26'!
typeOfColumn: aColumn
	^ NBSQLite3FFI current typeFor: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 17:18'!
valueOfColumn: aColumn
	^ self perform: (self typeOfColumn: aColumn) with: aColumn
! !
NBSQLite3FFI initialize!
